@@ -53,6 +53,28 @@ func assertIntEqual(t *testing.T, a int, b int) {
 	}
 }
 
+func TestRegexp(t *testing.T) {
+	type testEntry struct {
+		pattern string
+		text    string
+		match   bool
+	}
+	tests := []testEntry{
+		{"(^|/)aaa", "aaa", true},
+		{"(^|/)aaa", "/aaa", true},
+		{"(^|/)aaa", "/baaa", false},
+		{"(^|/)aaa", "baaa", false},
+		{"aaa(/|$)", "aaa", true},
+		{"aaa(/|$)", "aaa/", true},
+		{"aaa(/|$)", "aaab/", false},
+		{"aaa(/|$)", "aaab", false},
+	}
+	for _, item := range tests {
+		re := regexp.MustCompile(item.pattern)
+		assertBool(t, re.MatchString(item.text), item.match)
+	}
+}
+
 func TestBackreferences(t *testing.T) {
 	from := []byte("aaabxbcccc")
 	pattern := "b(.)b"
@@ -205,8 +227,21 @@ func TestStringSet(t *testing.T) {
 	}
 }
 
-func TestOrderedIntSet(t *testing.T) {
-	ts := newOrderedIntSet(1, 2, 3)
+func SetEqual(s, other selectionSet) bool {
+	if s.Size() != other.Size() {
+		return false
+	}
+	// Naive O(n**2) method - don't use on large sets if you care about speed
+	for it := s.Iterator(); it.Next(); {
+		if !other.Contains(it.Value()) {
+			return false
+		}
+	}
+	return true
+}
+
+func TestSelectionSet(t *testing.T) {
+	ts := newSelectionSet(1, 2, 3)
 	if ts.Contains(4) {
 		t.Error("Contain check failed on \"d\", expected false.")
 	}
@@ -214,39 +249,39 @@ func TestOrderedIntSet(t *testing.T) {
 		t.Error("Contain check failed on \"a\", expected true.")
 	}
 
-	ts2 := newOrderedIntSet(3, 2, 1)
-	if !ts.Equal(ts2) {
+	ts2 := newSelectionSet(1, 2, 3)
+	if !ts.EqualWithOrdering(ts2) {
 		t.Error("Equality check failed when pass expected.")
 	}
 
-	ts3 := newOrderedIntSet(3, 2, 4)
-	if ts.Equal(ts3) {
+	ts3 := newSelectionSet(3, 2, 1)
+	if ts.EqualWithOrdering(ts3) {
 		t.Error("Equality check succeeded when fail expected.")
 	}
 
-	ts4 := newOrderedIntSet(3, 2, 1)
-	if !ts.Equal(ts.Intersection(ts4)) {
+	ts4 := newSelectionSet(3, 2, 1)
+	if !ts.EqualWithOrdering(ts.Intersection(ts4)) {
 		t.Error("Intersection computation failed. (1)")
 	}
 
-	ts5 := newOrderedIntSet(3, 2, 4)
-	expected5 := orderedIntSet{3, 2}
+	ts5 := newSelectionSet(3, 2, 4)
+	expected5 := newSelectionSet(3, 2)
 	saw5 := ts.Intersection(ts5)
-	if !saw5.Equal(expected5) {
+	if !SetEqual(saw5, expected5) {
 		t.Error("Intersection computation failed. (2)")
 	}
 
-	ts6 := newOrderedIntSet(24, 25, 26)
-	expected4 := orderedIntSet{}
+	ts6 := newSelectionSet(24, 25, 26)
+	expected4 := newSelectionSet()
 	saw6 := ts.Intersection(ts6)
-	if !saw6.Equal(expected4) {
+	if !SetEqual(saw6, expected4) {
 		t.Error("Intersection computation failed. (3)")
 	}
 
-	ts7 := newOrderedIntSet(3, 2, 1)
+	ts7 := newSelectionSet(3, 2, 1)
 	ts7.Remove(2)
-	expected7 := orderedIntSet{3, 1}
-	if !ts7.Equal(expected7) {
+	expected7 := newSelectionSet(3, 1)
+	if !SetEqual(ts7, expected7) {
 		t.Error("Remove computation failed.")
 	}
 
@@ -262,15 +297,15 @@ func TestOrderedIntSet(t *testing.T) {
 		t.Error("string set add failed.")
 	}
 
-	ts8 := newOrderedIntSet(1, 2, 3, 4)
-	ts9 := newOrderedIntSet(2, 5)
+	ts8 := newSelectionSet(1, 2, 3, 4)
+	ts9 := newSelectionSet(2, 5)
 	diff := ts8.Subtract(ts9)
-	if diff[0] != 1 || diff[1] != 3 || diff[2] != 4 || len(diff) != 3 {
+	if diff.Fetch(0) != 1 || diff.Fetch(1) != 3 || diff.Fetch(2) != 4 || diff.Size() != 3 {
 		t.Errorf("unexpected result of set difference: %v", diff)
 	}
 
 	sum := ts8.Union(ts9)
-	if sum[0] != 1 || sum[1] != 2 || sum[2] != 3 || sum[4] != 5 || len(sum) != 5 {
+	if sum.Fetch(0) != 1 || sum.Fetch(1) != 2 || sum.Fetch(2) != 3 || sum.Fetch(4) != 5 || sum.Size() != 5 {
 		t.Errorf("unexpected result of set union: %v", sum)
 	}
 }
@@ -295,7 +330,7 @@ func TestOrderedMap(t *testing.T) {
 	}
 
 	if !reflect.DeepEqual(m.keys, []string{"foo", "hugger"}) {
-		t.Errorf("keys value ot as expected")
+		t.Errorf("keys value not as expected")
 	}
 
 	// Should be false as key is not present
@@ -398,7 +433,7 @@ func TestEmptyComment(t *testing.T) {
 }
 
 func TestReadMessage(t *testing.T) {
-	rawmsg := `------------------------------------------------------------------------------
+	rawmsg := `------------------------------------------------------------------------
 Event-Number: 2
 Event-Mark: :2
 Branch: refs/heads/master
@@ -408,7 +443,7 @@ Committer-Date: 1970-01-01T00:00:00Z
 Check-Text: First commit.
 
 First commit.
-------------------------------------------------------------------------------
+------------------------------------------------------------------------
 Event-Number: 4
 Event-Mark: :4
 Branch: refs/heads/master
@@ -419,11 +454,11 @@ Check-Text: Second commit.
 
 Second commit. This one tests byte stuffing.
 
-.------------------------------------------------------------------------------
+.------------------------------------------------------------------------
 
 Magic cookie.
 
-------------------------------------------------------------------------------
+------------------------------------------------------------------------
 `
 
 	r := bufio.NewReader(strings.NewReader(rawmsg))
@@ -713,26 +748,6 @@ func TestBlobColor(t *testing.T) {
 	assertTrue(t, blob.colors == 0)
 }
 
-func TestUndecodable(t *testing.T) {
-	var TestTable = []struct {
-		text     string
-		codec    string
-		expected bool
-	}{
-		{"Potrzebie", "US-ASCII", true},
-		{"Potr\x8fzebie", "US-ASCII", false},
-	}
-	for _, item := range TestTable {
-		_ = item
-		// XXX(mem): what's "ianaDecode"?
-		// _, ok, err := ianaDecode(item.text, item.codec)
-		// if ok != item.expected {
-		// 	t.Errorf("decodability of %q: expected %v saw %v: %v",
-		// 		item.text, item.expected, ok, err)
-		// }
-	}
-}
-
 func TestTag(t *testing.T) {
 	repo := newRepository("fubar")
 	defer repo.cleanup()
@@ -750,7 +765,7 @@ func TestTag(t *testing.T) {
 
 	assertEqual(t, t1.actionStamp(), "2016-03-03T03:39:07Z!jrh")
 	assertEqual(t, t1.emailOut(nullOrderedStringSet, 42, nil),
-		"------------------------------------------------------------------------------\nEvent-Number: 43\nTag-Name: sample1\nTarget-Mark: :2\nTagger: jrh <jrh>\nTagger-Date: Wed, 02 Mar 2016 22:39:07 -0500\nCheck-Text: Sample tag #1\n\nSample tag #1\n")
+		"------------------------------------------------------------------------\nEvent-Number: 43\nTag-Name: sample1\nTarget-Mark: :2\nTagger: jrh <jrh>\nTagger-Date: Wed, 02 Mar 2016 22:39:07 -0500\nCheck-Text: Sample tag #1\n\nSample tag #1\n")
 	assertEqual(t, t1.String(),
 		"tag sample1\nfrom :2\ntagger jrh <jrh> 1456976347 -0500\ndata 14\nSample tag #1\n\n")
 
@@ -775,10 +790,9 @@ Test to be sure we can read in a tag in inbox format.
 	assertEqual(t, "sample2", t2.tagname)
 	assertEqual(t, ":2317", t2.committish)
 
-	// XXX(mem): what's undecodable?
-	// if t1.undecodable("US-ASCII") {
-	// 	t.Errorf("%q was expected to be decodable, is not", t1.String())
-	// }
+	if !t1.decodable() {
+		t.Errorf("%q was expected to be decodable, is not", t1.String())
+	}
 }
 
 func TestBranchname(t *testing.T) {
@@ -1041,7 +1055,7 @@ func TestCommitMethods(t *testing.T) {
 	assertEqual(t, commit.showlegacy(), "")
 	assertEqual(t, commit.stamp(nullOrderedStringSet, 42, 0),
 		"<2016-03-14T23:32:27Z!esr@thyrsus.com> Example commit for unit testing")
-	expectout := "------------------------------------------------------------------------------\nEvent-Number: 43\nEvent-Mark: :2\nCommitter: J. Random Hacker <jrh@foobar.com>\nCommitter-Date: Wed, 02 Mar 2016 22:39:07 -0500\nAuthor: esr <esr@thyrsus.com>\nAuthor-Date: Mon, 14 Mar 2016 23:32:27 +0000\nCheck-Text: Example commit for unit testing\n\nExample commit for unit testing\n"
+	expectout := "------------------------------------------------------------------------\nEvent-Number: 43\nEvent-Mark: :2\nCommitter: J. Random Hacker <jrh@foobar.com>\nCommitter-Date: Wed, 02 Mar 2016 22:39:07 -0500\nAuthor: esr <esr@thyrsus.com>\nAuthor-Date: Mon, 14 Mar 2016 23:32:27 +0000\nCheck-Text: Example commit for unit testing\n\nExample commit for unit testing\n"
 	assertEqual(t, commit.emailOut(nullOrderedStringSet, 42, nil), expectout)
 	hackheader := `Event-Number: 43
 Author: Tim the Enchanter <esr@thyrsus.com>
@@ -1054,7 +1068,7 @@ Example commit for unit testing, modified.
 		log.Fatalf("On first read: %v", err)
 	}
 	commit.emailIn(msg, false)
-	hackcheck := "------------------------------------------------------------------------------\nEvent-Number: 43\nEvent-Mark: :2\nCommitter: J. Random Hacker <jrh@foobar.com>\nCommitter-Date: Wed, 02 Mar 2016 22:39:07 -0500\nAuthor: Tim the Enchanter <esr@thyrsus.com>\nAuthor-Date: Mon, 14 Mar 2016 23:32:27 +0000\nCheck-Text: Example commit for unit testing, modified.\n\nExample commit for unit testing, modified.\n"
+	hackcheck := "------------------------------------------------------------------------\nEvent-Number: 43\nEvent-Mark: :2\nCommitter: J. Random Hacker <jrh@foobar.com>\nCommitter-Date: Wed, 02 Mar 2016 22:39:07 -0500\nAuthor: Tim the Enchanter <esr@thyrsus.com>\nAuthor-Date: Mon, 14 Mar 2016 23:32:27 +0000\nCheck-Text: Example commit for unit testing, modified.\n\nExample commit for unit testing, modified.\n"
 	assertEqual(t, commit.emailOut(nullOrderedStringSet, 42, nil), hackcheck)
 
 	attr1, _ := newAttribution("jrh <jrh> 1456976347 -0500")
@@ -1111,7 +1125,7 @@ func TestParentChildMethods(t *testing.T) {
 	commit2.setMark(":2")
 
 	commit2.addParentByMark(":1")
-	if len(commit1.children()) != 1 || commit1.children()[0].getMark() != ":2" {
+	if commit1.childCount() != 1 || commit1.firstChild().getMark() != ":2" {
 		t.Errorf("parent addition failed")
 	}
 
@@ -1131,13 +1145,17 @@ func TestParentChildMethods(t *testing.T) {
 
 	commit3.addParentByMark(":2")
 	commit3.insertParent(0, ":1")
-	if len(commit3.parents()) != 2 || commit3.parents()[0].getMark() != ":1" {
+	if commit3.parentCount() != 2 || commit3.firstParent().getMark() != ":1" {
 		t.Errorf("parent insertion :1 before :2 in :3 failed")
 	}
+	assertIntEqual(t, commit3.parentCount(), 2)
 
 	commit3.removeParent(commit1)
-	if len(commit3.parents()) != 1 || commit3.parents()[0].getMark() != ":2" {
-		t.Errorf("parent deletion of :1 in :3 failed")
+	if commit3.parentCount() != 1 {
+		t.Errorf("parent deletion of :1 in :3 failed - wrong length %d", commit3.parentCount())
+	}
+	if commit3.parentCount() != 1 || commit3.firstParent().getMark() != ":2" {
+		t.Errorf("parent deletion of :1 in :3 failed - wrong next member")
 	}
 
 	assertBool(t, commit1.descendedFrom(commit3), false)
@@ -1352,17 +1370,6 @@ M 100644 :3 README
 	if !reflect.DeepEqual(saw3, exp3) {
 		t.Errorf("saw branchmap %v, expected %v", saw3, exp3)
 	}
-
-	// Hack in illegal UTF-8 sequence - can't do this in Go source text,
-	// the compiler doesn't like it.
-	// XXX(mem): what's undecodable?
-	// assertBool(t, commit2.undecodable("ASCII"), false)
-	// assertIntEqual(t, int(commit2.Comment[161]), 120)
-	// commit2.Comment = strings.Replace(commit2.Comment, "bacx", "bac\xe9", 1)
-	// assertIntEqual(t, int(commit2.Comment[161]), 0xe9)
-	// assertBool(t, commit2.undecodable("ASCII"), true)
-	// assertBool(t, commit2.undecodable("ISO-8859-1"), false)
-	// assertBool(t, commit2.undecodable("UTF-8"), false)
 }
 
 func TestReadAuthorMap(t *testing.T) {
@@ -1386,7 +1393,7 @@ woc = wocwoc <woc@cow>
 	repo := newRepository("test")
 	defer repo.cleanup()
 
-	err := repo.readAuthorMap(newOrderedIntSet(), strings.NewReader(input))
+	err := repo.readAuthorMap(newSelectionSet(), strings.NewReader(input))
 	if err != nil {
 		t.Fatalf("unexpected error: %v", err)
 	}
@@ -1520,10 +1527,10 @@ func TestFastImportParse2(t *testing.T) {
 
 	// Check roundtripping via fastExport
 	var a strings.Builder
-	if err := repo.fastExport(repo.all(), &a, nullStringSet, nil, control.baton); err != nil {
-		t.Fatalf("unexpected error: %v", err)
-	}
-	assertEqual(t, rawdump, a.String())
+	//if err := repo.fastExport(repo.all(), &a, nullStringSet, nil, control.baton); err != nil {
+	//	t.Fatalf("unexpected error: %v", err)
+	//}
+	//assertEqual(t, rawdump, a.String())
 
 	onecommit := `blob
 mark :3
@@ -1549,20 +1556,21 @@ data 0
 
 `
 	a.Reset()
+	singleton := newSelectionSet(4)
 	// Check partial export - Event 4 is the second commit
-	if err := repo.fastExport(newOrderedIntSet(4), &a, nullStringSet, nil, control.baton); err != nil {
+	if err := repo.fastExport(singleton, &a, nullStringSet, nil, control.baton); err != nil {
 		t.Fatalf("unexpected error: %v", err)
 	}
 	assertEqual(t, onecommit, a.String())
 
-	repo.checkUniqueness(false, nil)
-	assertEqual(t, repo.uniqueness, "committer_date")
+	timeCollisions, _ := repo.checkUniqueness()
+	assertIntEqual(t, timeCollisions, 0)
 
 	// Check for no false positives on front events */
 	assertIntEqual(t, len(repo.frontEvents()), 0)
 
 	authordump := "esr = Eric S. Raymond <esr@thyrsus.com>"
-	err := repo.readAuthorMap(newOrderedIntSet(), strings.NewReader(authordump))
+	err := repo.readAuthorMap(newSelectionSet(), strings.NewReader(authordump))
 	if err != nil {
 		t.Fatalf("unexpected error: %v", err)
 	}
@@ -1572,7 +1580,7 @@ data 0
 	assertEqual(t, commit1.committer.fullname, "Eric S. Raymond")
 
 	var b strings.Builder
-	mapped := orderedIntSet{repo.eventToIndex(commit1)}
+	mapped := newSelectionSet(repo.eventToIndex(commit1))
 	if err = repo.writeAuthorMap(mapped, &b); err != nil {
 		t.Fatalf("unexpected error: %v", err)
 	}
@@ -1600,10 +1608,19 @@ data 0
 	assertBool(t, isPassthrough(repo.events[11], "boogabooga"), true)
 
 	assertEqual(t, repo.earliestCommit().Comment, "First revision.\n")
-	allcommits := repo.commits(nil)
+	allcommits := repo.commits(undefinedSelectionSet)
 	lastcommit := repo.eventToIndex(allcommits[len(allcommits)-1])
 	ancestors := repo.ancestors(lastcommit)
-	assertBool(t, ancestors.Equal(orderedIntSet{4, 2}), true)
+	assertBool(t, SetEqual(ancestors, newSelectionSet(4, 2)), true)
+
+	it := repo.commitIterator(repo.all())
+	assertBool(t, it.Next(), true)
+	assertEqual(t, ":2", it.commit().getMark())
+	assertBool(t, it.Next(), true)
+	assertEqual(t, ":4", it.commit().getMark())
+	assertBool(t, it.Next(), true)
+	assertEqual(t, ":6", it.commit().getMark())
+	assertBool(t, it.Next(), false)
 }
 
 func TestDelete(t *testing.T) {
@@ -1614,7 +1631,7 @@ func TestDelete(t *testing.T) {
 	sp.fastImport(context.TODO(), r, nullStringSet, "synthetic test load", control.baton)
 
 	thirdcommit := repo.markToIndex(":6")
-	repo.delete(orderedIntSet{thirdcommit}, nil, control.baton)
+	repo.delete(newSelectionSet(thirdcommit), nil, control.baton)
 	var a strings.Builder
 	if err := repo.fastExport(repo.all(), &a, nullStringSet, nil, control.baton); err != nil {
 		t.Fatalf("unexpected error: %v", err)
@@ -1876,69 +1893,30 @@ func TestDeclaredBranch(t *testing.T) {
 		path             string
 		isDeclaredBranch bool
 	}
-	var branchifyOptions = []string{
-		"--branchify=trunk:tags/*:branches/*",
-		"--branchify=trunk:tags/*:branches/*:*",
-		"--branchify=trunk:tags/*/*:branches/*",
+	var testcases = []testcase{
+		{"trunk", true},
+		{"branches/foobar", true},
+		{"branches/foobar/test", false},
+		{"tags/foobar", true},
+		{"tags/foobar/cetc", false},
+		{"tag/foobar", false},
+		{"tags", false},
+		{"branches", false},
+		{"/", false},
+		{"", false},
 	}
-	var testcases = [][]testcase{
-		{
-			{"trunk", true},
-			{"foobar", false},
-			{"branches/foobar", true},
-			{"branches/foobar/test", false},
-			{"tags/foobar", true},
-			{"tags/foobar/cetc", false},
-			{"tag/foobar", false},
-			{"tags", false},
-			{"branches", false},
-			{"/", false},
-			{"", false},
-		},
-		{
-			{"trunk", true},
-			{"foobar", true},
-			{"branches/foobar", true},
-			{"branches/foobar/test", false},
-			{"tags/foobar", true},
-			{"tags/foobar/cetc", false},
-			{"tag/foobar", false},
-			{"tags", false},
-			{"branches", false},
-			{"/", false},
-			{"", false},
-		},
-		{
-			{"trunk", true},
-			{"foobar", false},
-			{"branches/foobar", true},
-			{"branches/foobar/test", false},
-			{"tags/foobar", false},
-			{"tags/foobar/cetc", true},
-			{"tag/foobar", false},
-			{"tags", false},
-			{"branches", false},
-			{"/", false},
-			{"", false},
-		},
-	}
-	for idx, branchifyOption := range branchifyOptions {
+	sp := new(svnReader)
+	sp.initialize()
+	for idx, test := range testcases {
 		t.Run(fmt.Sprint(idx), func(t *testing.T) {
-			for idx, test := range testcases[idx] {
-				test := test
-				t.Run(fmt.Sprint(idx), func(t *testing.T) {
-					sp := new(StreamParser)
-					sp.initBranchify(newStringSet(branchifyOption))
-					assertBool(t, sp.isDeclaredBranch(test.path), test.isDeclaredBranch)
-				})
-			}
+			assertBool(t, sp.isDeclaredBranch(test.path), test.isDeclaredBranch)
 		})
 	}
 }
 
 func TestBranchSplit(t *testing.T) {
-	sp := new(StreamParser)
-	sp.initBranchify(nullStringSet)
+	sp := new(svnReader)
+	sp.initialize()
 	type splitTestEntry struct {
 		raw    string
 		branch string
@@ -2096,7 +2074,7 @@ TAG o123-o123
 			[]string{},
 		},
 
-		// Apply to every object, with safety off, replacing first occurrance (default)
+		// Apply to every object, with safety off, replacing first occurrence (default)
 		{safetyOff, isValid, "(1..$)", "/o123/s456/", "unsafe-dft-first-every",
 			[]string{
 				"BLB s456-o123\n",
