@@ -1,7 +1,10 @@
 package main
 
-// The event selector. Exports one class, which is used
-// to compile and evaluate selection expressions.
+// The event selector. Exports one class, selectionSet.  Adds to the
+// Reposurgeon class two methods: parseSelectionSet() that is used to
+// compile selection-expression syntax to a matching machine, and
+// evalSelectionSet() that is uses to produce a selection set using
+// the machine.
 //
 // Copyright by Eric S. Raymond
 // SPDX-License-Identifier: BSD-2-Clause
@@ -9,6 +12,7 @@ package main
 import (
 	"fmt"
 	"io"
+	"math"
 	"reflect"
 	"regexp"
 	"sort"
@@ -16,8 +20,207 @@ import (
 	"strings"
 	"unicode"
 	"unicode/utf8"
+
+	orderedset "github.com/emirpasic/gods/sets/linkedhashset"
 )
 
+type selectionSet struct{ set *orderedset.Set }
+
+type selectionSetIt struct{ orderedset.Iterator }
+
+func newSelectionSet(x ...int) selectionSet {
+	s := orderedset.New()
+	for _, i := range x {
+		s.Add(i)
+	}
+	return selectionSet{s}
+}
+
+func (s selectionSet) isDefined() bool {
+	return s.set != nil
+}
+
+var undefinedSelectionSet selectionSet // Do not add to this, havoc would ensue
+
+func (s selectionSet) Fetch(idx int) int {
+	it := s.set.Iterator()
+	for it.Next() {
+		if idx == 0 {
+			break
+		}
+		idx--
+	}
+	return it.Value().(int)
+}
+
+func (x *selectionSetIt) Value() int {
+	return x.Iterator.Value().(int)
+}
+
+func (s selectionSet) Size() int {
+	if s.set == nil {
+		return 0
+	}
+	return s.set.Size()
+}
+
+func (s selectionSet) Iterator() selectionSetIt {
+	return selectionSetIt{Iterator: s.set.Iterator()}
+}
+
+func (s selectionSet) Values() []int {
+	v := make([]int, s.Size())
+	it := s.Iterator()
+	for it.Next() {
+		v[it.Index()] = it.Value()
+	}
+	return v
+}
+
+func (s selectionSet) Contains(x int) bool {
+	return s.set != nil && s.set.Contains(x)
+}
+
+func (s *selectionSet) Remove(x int) bool {
+	if s.Contains(x) {
+		s.set.Remove(x)
+		return true
+	}
+	return false
+}
+
+func (s *selectionSet) Add(x int) {
+	if s.set == nil {
+		s.set = orderedset.New(x)
+		return
+	}
+	s.set.Add(x)
+}
+
+func (s selectionSet) Subtract(other selectionSet) selectionSet {
+	p := orderedset.New()
+	it := s.set.Iterator()
+	for it.Next() {
+		if !other.set.Contains(it.Value()) {
+			p.Add(it.Value())
+		}
+	}
+
+	return selectionSet{p}
+}
+
+func (s selectionSet) Intersection(other selectionSet) selectionSet {
+	p := orderedset.New()
+	it := s.set.Iterator()
+	for it.Next() {
+		if other.set.Contains(it.Value()) {
+			p.Add(it.Value())
+		}
+	}
+	return selectionSet{p}
+}
+
+func (s selectionSet) Union(other selectionSet) selectionSet {
+	p := orderedset.New(s.set.Values()...)
+	p.Add(other.set.Values()...)
+	return selectionSet{p}
+}
+
+func (s selectionSet) EqualWithOrdering(other selectionSet) bool {
+	if s.Size() != other.Size() {
+		return false
+	}
+	sl := s.Values()
+	ol := other.Values()
+	for i := range sl {
+		if sl[i] != ol[i] {
+			return false
+		}
+	}
+	return true
+}
+
+func (s selectionSet) Min() int {
+	var min = math.MaxInt32
+	for it := s.Iterator(); it.Next(); {
+		v := it.Value()
+		if v < min {
+			min = v
+		}
+	}
+	return min
+}
+
+func (s *selectionSet) Sort() {
+	v := s.set.Values()
+	sort.Slice(v, func(i, j int) bool { return v[i].(int) < v[j].(int) })
+	s.set = orderedset.New(v...)
+}
+
+func (s *selectionSet) Pop() int {
+	x := (*s).Values()[(*s).Size()-1]
+	(*s).Remove(x)
+	return x
+}
+
+func (s selectionSet) String() string {
+	var b strings.Builder
+	b.WriteRune('[')
+	it := s.Iterator()
+	for it.Next() {
+		if it.Index() > 0 {
+			b.WriteString(", ")
+		}
+		b.WriteString(strconv.Itoa(it.Value()))
+	}
+	b.WriteRune(']')
+	return b.String()
+}
+
+type skipIt struct {
+	repo *Repository
+	iter selectionSetIt
+	pred func(event Event) bool
+}
+
+func (repo Repository) skipIterator(sel selectionSet, pred func(event Event) bool) skipIt {
+	var it skipIt
+	it.repo = &repo
+	it.iter = sel.Iterator()
+	it.pred = pred
+	return it
+}
+
+func (s skipIt) Index() int {
+	return s.iter.Index()
+}
+
+func (s skipIt) Value() int {
+	return s.iter.Value()
+}
+
+func (s *skipIt) Next() bool {
+	for s.iter.Next() {
+		if s.pred(s.repo.events[s.iter.Value()]) {
+			return true
+		}
+	}
+	return false
+}
+
+func (repo Repository) commitIterator(sel selectionSet) skipIt {
+	return repo.skipIterator(sel, func(event Event) bool { return event.isCommit() })
+}
+
+func (s skipIt) commit() *Commit {
+	return s.repo.events[s.iter.Value()].(*Commit)
+}
+
+// unclean tells us how to detect ungitified comments.  Check for Git
+// conventions - require a spacer line after first \n if multiline,
+// subject line no more than 50 chars, body lines no more than 72.
+var unclean = regexp.MustCompile("^[^\n]*\n[^\n]|^.{51,}\n|\n.{73,}")
+
 //
 // The selection-language parsing code starts here.
 //
@@ -31,7 +234,7 @@ func (rs *Reposurgeon) parseSelectionSet(line string) (machine selEvaluator, res
 	return rs.imp().compile(line)
 }
 
-func (rs *Reposurgeon) evalSelectionSet(machine selEvaluator, repo *Repository) []int {
+func (rs *Reposurgeon) evalSelectionSet(machine selEvaluator, repo *Repository) selectionSet {
 	state := rs.evalState(len(repo.events))
 	defer state.release()
 	return rs.imp().evaluate(machine, state)
@@ -51,7 +254,7 @@ func (rs *Reposurgeon) isNamed(s string) (result bool) {
 	}(&result)
 	repo := rs.chosen()
 	if repo != nil {
-		result = repo.named(s) != nil
+		result = repo.named(s).isDefined()
 	}
 	return
 }
@@ -67,7 +270,7 @@ func (rs *Reposurgeon) parseExpression() selEvaluator {
 		}
 		rs.pop()
 		orig := value
-		value = func(x selEvalState, s *fastOrderedIntSet) *fastOrderedIntSet {
+		value = func(x selEvalState, s selectionSet) selectionSet {
 			return rs.evalNeighborhood(x, s, orig)
 		}
 	}
@@ -75,20 +278,24 @@ func (rs *Reposurgeon) parseExpression() selEvaluator {
 }
 
 func (rs *Reposurgeon) evalNeighborhood(state selEvalState,
-	preselection *fastOrderedIntSet, subject selEvaluator) *fastOrderedIntSet {
+	preselection selectionSet, subject selEvaluator) selectionSet {
 	value := subject(state, preselection)
-	addSet := newFastOrderedIntSet()
-	removeSet := newFastOrderedIntSet()
+	addSet := newSelectionSet()
+	removeSet := newSelectionSet()
 	it := value.Iterator()
 	for it.Next() {
 		ei := it.Value()
 		event := rs.chosen().events[ei]
 		if c, ok := event.(*Commit); ok {
 			for _, parent := range c.parents() {
-				addSet.Add(rs.chosen().markToIndex(parent.getMark()))
+				if parentCommit, ok := parent.(*Commit); ok {
+					addSet.Add(rs.chosen().markToIndex(parentCommit.getMark()))
+				}
 			}
 			for _, child := range c.children() {
-				addSet.Add(rs.chosen().markToIndex(child.getMark()))
+				if childCommit, ok := child.(*Commit); ok {
+					addSet.Add(rs.chosen().markToIndex(childCommit.getMark()))
+				}
 			}
 		} else if _, ok := event.(*Blob); ok {
 			removeSet.Add(ei) // Don't select the blob itself
@@ -117,7 +324,7 @@ func (rs *Reposurgeon) evalNeighborhood(state selEvalState,
 	}
 	value = value.Union(addSet)
 	value = value.Subtract(removeSet)
-	value = value.Sort()
+	value.Sort()
 	return value
 }
 
@@ -172,19 +379,19 @@ func (rs *Reposurgeon) parsePathset() selEvaluator {
 		if err != nil {
 			panic(throw("command", "invalid regular expression %s", matcher))
 		}
-		return func(x selEvalState, s *fastOrderedIntSet) *fastOrderedIntSet {
+		return func(x selEvalState, s selectionSet) selectionSet {
 			return rs.evalPathsetRegex(x, s, search, flags)
 		}
 	}
-	return func(x selEvalState, s *fastOrderedIntSet) *fastOrderedIntSet {
+	return func(x selEvalState, s selectionSet) selectionSet {
 		return rs.evalPathset(x, s, matcher)
 	}
 }
 
 // Resolve a path regex to the set of commits that refer to it.
 func (rs *Reposurgeon) evalPathsetRegex(state selEvalState,
-	preselection *fastOrderedIntSet, search *regexp.Regexp,
-	flags orderedStringSet) *fastOrderedIntSet {
+	preselection selectionSet, search *regexp.Regexp,
+	flags orderedStringSet) selectionSet {
 	if flags.Contains("c") {
 		return rs.evalPathsetFull(state, preselection,
 			search, flags.Contains("a"))
@@ -197,7 +404,7 @@ func (rs *Reposurgeon) evalPathsetRegex(state selEvalState,
 	type vendPaths interface {
 		paths(orderedStringSet) orderedStringSet
 	}
-	hits := newFastOrderedIntSet()
+	hits := newSelectionSet()
 	events := rs.chosen().events
 	it := preselection.Iterator()
 	for it.Next() {
@@ -222,11 +429,11 @@ func (rs *Reposurgeon) evalPathsetRegex(state selEvalState,
 
 // Resolve a path name to the set of commits that refer to it.
 func (rs *Reposurgeon) evalPathset(state selEvalState,
-	preselection *fastOrderedIntSet, matcher string) *fastOrderedIntSet {
+	preselection selectionSet, matcher string) selectionSet {
 	type vendPaths interface {
 		paths(orderedStringSet) orderedStringSet
 	}
-	hits := newFastOrderedIntSet()
+	hits := newSelectionSet()
 	events := rs.chosen().events
 	it := preselection.Iterator()
 	for it.Next() {
@@ -239,8 +446,8 @@ func (rs *Reposurgeon) evalPathset(state selEvalState,
 }
 
 func (rs *Reposurgeon) evalPathsetFull(state selEvalState,
-	preselection *fastOrderedIntSet, matchCond *regexp.Regexp,
-	matchAll bool) *fastOrderedIntSet {
+	preselection selectionSet, matchCond *regexp.Regexp,
+	matchAll bool) selectionSet {
 	// Try to match a regex in the trees. For each commit we remember
 	// only the part of the tree that matches the regex. In most cases
 	// it is a lot less memory and CPU hungry than running regexes on
@@ -251,7 +458,7 @@ func (rs *Reposurgeon) evalPathsetFull(state selEvalState,
 		match = func(s string) bool { return !matchCond.MatchString(s) }
 	}
 	matchTrees := make(map[string]*PathMap)
-	result := newFastOrderedIntSet()
+	result := newSelectionSet()
 	lastEvent := selMax(preselection)
 	for i, event := range rs.chosen().events {
 		c, ok := event.(*Commit)
@@ -262,10 +469,10 @@ func (rs *Reposurgeon) evalPathsetFull(state selEvalState,
 			break
 		}
 		var tree *PathMap
-		parents := c.parents()
-		if len(parents) == 0 {
+		if !c.hasParents() {
 			tree = newPathMap()
 		} else {
+			parents := c.parents()
 			parentTree, ok := matchTrees[parents[0].getMark()]
 			if !ok {
 				panic(fmt.Sprintf("commit tree missing: %s",
@@ -314,6 +521,7 @@ func (rs *Reposurgeon) hasReference(event Event) bool {
 func (rs *Reposurgeon) visibilityTypeletters() map[rune]func(int) bool {
 	type decodable interface {
 		decodable() bool
+		ascii() bool
 	}
 	type alldel interface {
 		alldeletes(...rune) bool
@@ -325,20 +533,21 @@ func (rs *Reposurgeon) visibilityTypeletters() map[rune]func(int) bool {
 	return map[rune]func(int) bool{
 		'B': func(i int) bool { _, ok := e(i).(*Blob); return ok },
 		'C': func(i int) bool { _, ok := e(i).(*Commit); return ok },
-		'T': func(i int) bool { _, ok := e(i).(*Tag); return ok },
-		'R': func(i int) bool { _, ok := e(i).(*Reset); return ok },
-		'P': func(i int) bool { _, ok := e(i).(*Passthrough); return ok },
+		'D': func(i int) bool { p, ok := e(i).(alldel); return ok && p.alldeletes() },
+		'F': func(i int) bool { c, ok := e(i).(*Commit); return ok && c.childCount() > 1 },
 		'H': func(i int) bool { c, ok := e(i).(*Commit); return ok && !c.hasChildren() },
+		'I': func(i int) bool { p, ok := e(i).(decodable); return ok && !p.decodable() },
+		'J': func(i int) bool { p, ok := e(i).(decodable); return ok && !p.ascii() },
+		'L': func(i int) bool { c, ok := e(i).(*Commit); return ok && unclean.MatchString(c.Comment) },
+		'M': func(i int) bool { c, ok := e(i).(*Commit); return ok && c.parentCount() > 1 },
+		'N': func(i int) bool { return rs.hasReference(e(i)) },
 		'O': func(i int) bool { c, ok := e(i).(*Commit); return ok && !c.hasParents() },
+		'P': func(i int) bool { _, ok := e(i).(*Passthrough); return ok },
+		'Q': func(i int) bool { c, ok := e(i).(*Commit); return ok && c.hasColor(colorQSET) },
+		'R': func(i int) bool { _, ok := e(i).(*Reset); return ok },
+		'T': func(i int) bool { _, ok := e(i).(*Tag); return ok },
 		'U': func(i int) bool { c, ok := e(i).(*Commit); return ok && c.hasCallouts() },
 		'Z': func(i int) bool { c, ok := e(i).(*Commit); return ok && len(c.operations()) == 0 },
-		'M': func(i int) bool { c, ok := e(i).(*Commit); return ok && len(c.parents()) > 1 },
-		'F': func(i int) bool { c, ok := e(i).(*Commit); return ok && len(c.children()) > 1 },
-		'L': func(i int) bool { c, ok := e(i).(*Commit); return ok && unclean.MatchString(c.Comment) },
-		'Q': func(i int) bool { c, ok := e(i).(*Commit); return ok && c.getDelFlag() },
-		'I': func(i int) bool { p, ok := e(i).(decodable); return ok && !p.decodable() },
-		'D': func(i int) bool { p, ok := e(i).(alldel); return ok && p.alldeletes() },
-		'N': func(i int) bool { return rs.hasReference(e(i)) },
 	}
 }
 
@@ -369,7 +578,7 @@ func (rs *Reposurgeon) parseAtom() selEvaluator {
 		markref := markRE.FindString(rs.line)
 		if len(markref) > 0 {
 			rs.line = rs.line[len(markref):]
-			selection = func(x selEvalState, s *fastOrderedIntSet) *fastOrderedIntSet {
+			selection = func(x selEvalState, s selectionSet) selectionSet {
 				return rs.evalAtomMark(x, s, markref)
 			}
 		} else if rs.peek() == ':' {
@@ -382,7 +591,7 @@ func (rs *Reposurgeon) parseAtom() selEvaluator {
 			}
 			ref := rs.line[:closer]
 			rs.line = rs.line[closer+1:]
-			selection = func(x selEvalState, s *fastOrderedIntSet) *fastOrderedIntSet {
+			selection = func(x selEvalState, s selectionSet) selectionSet {
 				return rs.evalAtomRef(x, s, ref)
 			}
 		}
@@ -391,28 +600,28 @@ func (rs *Reposurgeon) parseAtom() selEvaluator {
 }
 
 func (rs *Reposurgeon) evalAtomMark(state selEvalState,
-	preselection *fastOrderedIntSet, markref string) *fastOrderedIntSet {
+	preselection selectionSet, markref string) selectionSet {
 	events := rs.chosen().events
 	for i := 0; i < state.nItems(); i++ {
 		e := events[i]
 		if val, ok := getAttr(e, "mark"); ok && val == markref {
-			return newFastOrderedIntSet(i)
+			return newSelectionSet(i)
 		} else if val, ok := getAttr(e, "committish"); ok && val == markref {
-			return newFastOrderedIntSet(i)
+			return newSelectionSet(i)
 		}
 	}
 	panic(throw("command", "mark %s not found.", markref))
 }
 
 func (rs *Reposurgeon) evalAtomRef(state selEvalState,
-	preselection *fastOrderedIntSet, ref string) *fastOrderedIntSet {
-	selection := newFastOrderedIntSet()
+	preselection selectionSet, ref string) selectionSet {
+	selection := newSelectionSet()
 	lookup := rs.chosen().named(ref)
-	if lookup != nil {
+	if lookup.isDefined() {
 		// Choose to include *all* commits matching the date.
 		// Alas, results in unfortunate behavior when a date
 		// with multiple commits ends a span.
-		selection = selection.Union(newFastOrderedIntSet(lookup...))
+		selection = selection.Union(lookup)
 	} else {
 		panic(throw("command", "couldn't match a name at <%s>", ref))
 	}
@@ -421,9 +630,9 @@ func (rs *Reposurgeon) evalAtomRef(state selEvalState,
 
 // Perform a text search of items.
 func (rs *Reposurgeon) evalTextSearch(state selEvalState,
-	preselection *fastOrderedIntSet,
-	search *regexp.Regexp, modifiers string) *fastOrderedIntSet {
-	matchers := newFastOrderedIntSet()
+	preselection selectionSet,
+	search *regexp.Regexp, modifiers string) selectionSet {
+	matchers := newSelectionSet()
 	// values ("author", "Branch", etc.) in 'searchableAttrs' and keys in
 	// 'extractors' (below) must exactly match spelling and case of fields
 	// in structures being interrogated since reflection is used both to
@@ -439,7 +648,7 @@ func (rs *Reposurgeon) evalTextSearch(state selEvalState,
 		'r': "committish", // tag or reset
 		'p': "text",       // passthrough
 		't': "tagger",     // tag
-		'n': "name",       // tag
+		'n': "tagname",    // tag
 	}
 	var searchIn []string
 	for _, v := range searchableAttrs {
@@ -474,7 +683,7 @@ func (rs *Reposurgeon) evalTextSearch(state selEvalState,
 			}
 			panic(fmt.Sprintf(`no "tagger" in %T`, e))
 		},
-		"name": func(e Event) string { return exattr(e, "name") },
+		"tagname": func(e Event) string { return exattr(e, "tagname") },
 	}
 	checkAuthors := false
 	checkBlobs := false
@@ -546,70 +755,70 @@ func (rs *Reposurgeon) evalTextSearch(state selEvalState,
 
 func (rs *Reposurgeon) functions() map[string]selEvaluator {
 	return map[string]selEvaluator{
-		"chn": func(state selEvalState, subarg *fastOrderedIntSet) *fastOrderedIntSet {
+		"chn": func(state selEvalState, subarg selectionSet) selectionSet {
 			return rs.chnHandler(state, subarg)
 		},
-		"dsc": func(state selEvalState, subarg *fastOrderedIntSet) *fastOrderedIntSet {
+		"dsc": func(state selEvalState, subarg selectionSet) selectionSet {
 			return rs.dscHandler(state, subarg)
 		},
-		"par": func(state selEvalState, subarg *fastOrderedIntSet) *fastOrderedIntSet {
+		"par": func(state selEvalState, subarg selectionSet) selectionSet {
 			return rs.parHandler(state, subarg)
 		},
-		"anc": func(state selEvalState, subarg *fastOrderedIntSet) *fastOrderedIntSet {
+		"anc": func(state selEvalState, subarg selectionSet) selectionSet {
 			return rs.ancHandler(state, subarg)
 		},
 	}
 }
 
 // All children of commits in the selection set.
-func (rs *Reposurgeon) chnHandler(state selEvalState, subarg *fastOrderedIntSet) *fastOrderedIntSet {
+func (rs *Reposurgeon) chnHandler(state selEvalState, subarg selectionSet) selectionSet {
 	return rs.accumulateCommits(subarg,
 		func(c *Commit) []CommitLike { return c.children() }, false)
 }
 
 // All descendants of a selection set, recursively.
-func (rs *Reposurgeon) dscHandler(state selEvalState, subarg *fastOrderedIntSet) *fastOrderedIntSet {
+func (rs *Reposurgeon) dscHandler(state selEvalState, subarg selectionSet) selectionSet {
 	return rs.accumulateCommits(subarg,
 		func(c *Commit) []CommitLike { return c.children() }, true)
 }
 
 // All parents of a selection set.
-func (rs *Reposurgeon) parHandler(state selEvalState, subarg *fastOrderedIntSet) *fastOrderedIntSet {
+func (rs *Reposurgeon) parHandler(state selEvalState, subarg selectionSet) selectionSet {
 	return rs.accumulateCommits(subarg,
 		func(c *Commit) []CommitLike { return c.parents() }, false)
 }
 
 // All ancestors of a selection set, recursively.
-func (rs *Reposurgeon) ancHandler(state selEvalState, subarg *fastOrderedIntSet) *fastOrderedIntSet {
+func (rs *Reposurgeon) ancHandler(state selEvalState, subarg selectionSet) selectionSet {
 	return rs.accumulateCommits(subarg,
 		func(c *Commit) []CommitLike { return c.parents() }, true)
 }
 
 type selEvalState interface {
 	nItems() int
-	allItems() *fastOrderedIntSet
+	allItems() selectionSet
 	release()
 }
 
-type selEvaluator func(selEvalState, *fastOrderedIntSet) *fastOrderedIntSet
+type selEvaluator func(selEvalState, selectionSet) selectionSet
 
 type selParser interface {
 	compile(line string) (selEvaluator, string)
-	evaluate(selEvaluator, selEvalState) []int
-	parse(string, selEvalState) ([]int, string)
+	evaluate(selEvaluator, selEvalState) selectionSet
+	parse(string, selEvalState) (selectionSet, string)
 	parseExpression() selEvaluator
 	parseDisjunct() selEvaluator
-	evalDisjunct(selEvalState, *fastOrderedIntSet, selEvaluator, selEvaluator) *fastOrderedIntSet
+	evalDisjunct(selEvalState, selectionSet, selEvaluator, selEvaluator) selectionSet
 	parseConjunct() selEvaluator
-	evalConjunct(selEvalState, *fastOrderedIntSet, selEvaluator, selEvaluator) *fastOrderedIntSet
+	evalConjunct(selEvalState, selectionSet, selEvaluator, selEvaluator) selectionSet
 	parseTerm() selEvaluator
-	evalTermNegate(selEvalState, *fastOrderedIntSet, selEvaluator) *fastOrderedIntSet
+	evalTermNegate(selEvalState, selectionSet, selEvaluator) selectionSet
 	parseVisibility() selEvaluator
-	evalVisibility(selEvalState, *fastOrderedIntSet, string) *fastOrderedIntSet
+	evalVisibility(selEvalState, selectionSet, string) selectionSet
 	parsePolyrange() selEvaluator
 	polyrangeInitials() string
 	possiblePolyrange() bool
-	evalPolyrange(selEvalState, *fastOrderedIntSet, []selEvaluator) *fastOrderedIntSet
+	evalPolyrange(selEvalState, selectionSet, []selEvaluator) selectionSet
 	parseAtom() selEvaluator
 	parseTextSearch() selEvaluator
 	parseFuncall() selEvaluator
@@ -638,8 +847,8 @@ func (p *SelectionParser) release() { p.nitems = 0 }
 
 func (p *SelectionParser) nItems() int { return p.nitems }
 
-func (p *SelectionParser) allItems() *fastOrderedIntSet {
-	s := newFastOrderedIntSet()
+func (p *SelectionParser) allItems() selectionSet {
+	s := newSelectionSet()
 	for i := 0; i < p.nitems; i++ {
 		s.Add(i)
 	}
@@ -669,15 +878,15 @@ func (p *SelectionParser) compile(line string) (selEvaluator, string) {
 }
 
 // evaluate evaluates a pre-compiled selection query against item list
-func (p *SelectionParser) evaluate(machine selEvaluator, state selEvalState) []int {
+func (p *SelectionParser) evaluate(machine selEvaluator, state selEvalState) selectionSet {
 	if machine == nil {
-		return nil
+		return undefinedSelectionSet
 	}
-	return machine(state, p.allItems()).Values()
+	return newSelectionSet(machine(state, p.allItems()).Values()...)
 }
 
 // parse parses selection and returns remainder of line with selection removed
-func (p *SelectionParser) parse(line string, state selEvalState) ([]int, string) {
+func (p *SelectionParser) parse(line string, state selEvalState) (selectionSet, string) {
 	machine, rest := p.imp().compile(line)
 	return p.imp().evaluate(machine, state), rest
 }
@@ -722,7 +931,7 @@ func (p *SelectionParser) parseDisjunct() selEvaluator {
 			break
 		}
 		op1 := op
-		op = func(x selEvalState, s *fastOrderedIntSet) *fastOrderedIntSet {
+		op = func(x selEvalState, s selectionSet) selectionSet {
 			return p.imp().evalDisjunct(x, s, op1, op2)
 		}
 	}
@@ -731,13 +940,13 @@ func (p *SelectionParser) parseDisjunct() selEvaluator {
 
 // evalDisjunct evaluates a disjunctive expression
 func (p *SelectionParser) evalDisjunct(state selEvalState,
-	preselection *fastOrderedIntSet, op1, op2 selEvaluator) *fastOrderedIntSet {
-	selected := newFastOrderedIntSet()
+	preselection selectionSet, op1, op2 selEvaluator) selectionSet {
+	selected := newSelectionSet()
 	conjunct := op1(state, preselection)
-	if conjunct != nil {
+	if conjunct.isDefined() {
 		selected = selected.Union(conjunct)
 		conjunct = op2(state, preselection)
-		if conjunct != nil {
+		if conjunct.isDefined() {
 			selected = selected.Union(conjunct)
 		}
 	}
@@ -762,7 +971,7 @@ func (p *SelectionParser) parseConjunct() selEvaluator {
 			break
 		}
 		op1 := op
-		op = func(x selEvalState, s *fastOrderedIntSet) *fastOrderedIntSet {
+		op = func(x selEvalState, s selectionSet) selectionSet {
 			return p.imp().evalConjunct(x, s, op1, op2)
 		}
 	}
@@ -771,12 +980,12 @@ func (p *SelectionParser) parseConjunct() selEvaluator {
 
 // evalConjunct evaluates a conjunctive expression
 func (p *SelectionParser) evalConjunct(state selEvalState,
-	preselection *fastOrderedIntSet, op1, op2 selEvaluator) *fastOrderedIntSet {
+	preselection selectionSet, op1, op2 selEvaluator) selectionSet {
 	// assign term before intersecting with preselection so
 	// that the order specified by the user's first term is
 	// preserved
 	conjunct := op1(state, preselection)
-	if conjunct == nil {
+	if !conjunct.isDefined() {
 		conjunct = preselection
 	} else {
 		// this line is necessary if the user specified only
@@ -784,7 +993,7 @@ func (p *SelectionParser) evalConjunct(state selEvalState,
 		// preselection
 		conjunct = conjunct.Intersection(preselection)
 		term := op2(state, preselection)
-		if term != nil {
+		if term.isDefined() {
 			conjunct = conjunct.Intersection(term)
 		}
 	}
@@ -797,7 +1006,7 @@ func (p *SelectionParser) parseTerm() selEvaluator {
 	if p.peek() == '~' {
 		p.pop()
 		op := p.imp().parseExpression()
-		term = func(x selEvalState, s *fastOrderedIntSet) *fastOrderedIntSet {
+		term = func(x selEvalState, s selectionSet) selectionSet {
 			return p.imp().evalTermNegate(x, s, op)
 		}
 	} else if p.peek() == '(' {
@@ -825,12 +1034,12 @@ func (p *SelectionParser) parseTerm() selEvaluator {
 }
 
 func (p *SelectionParser) evalTermNegate(state selEvalState,
-	preselection *fastOrderedIntSet, op selEvaluator) *fastOrderedIntSet {
+	preselection selectionSet, op selEvaluator) selectionSet {
 	if op == nil {
 		panic(throw("command", "inner expression to be negated is invalid"))
 	}
 	negated := op(state, state.allItems())
-	remainder := newFastOrderedIntSet()
+	remainder := newSelectionSet()
 	for i, n := 0, state.nItems(); i < n; i++ {
 		if !negated.Contains(i) {
 			remainder.Add(i)
@@ -869,7 +1078,7 @@ func (p *SelectionParser) parseVisibility() selEvaluator {
 		if !strings.ContainsRune("()|& ", p.peek()) {
 			panic(throw("command", "garbled type mask at '%s'", p.line))
 		}
-		visibility = func(x selEvalState, s *fastOrderedIntSet) *fastOrderedIntSet {
+		visibility = func(x selEvalState, s selectionSet) selectionSet {
 			return p.imp().evalVisibility(x, s, visible)
 		}
 	}
@@ -878,7 +1087,7 @@ func (p *SelectionParser) parseVisibility() selEvaluator {
 
 // evalVisibility evaluates a visibility spec
 func (p *SelectionParser) evalVisibility(state selEvalState,
-	preselection *fastOrderedIntSet, visible string) *fastOrderedIntSet {
+	preselection selectionSet, visible string) selectionSet {
 	type typelettersGetter interface {
 		visibilityTypeletters() map[rune]func(int) bool
 	}
@@ -887,7 +1096,7 @@ func (p *SelectionParser) evalVisibility(state selEvalState,
 	for i, r := range visible {
 		predicates[i] = typeletters[r]
 	}
-	visibility := newFastOrderedIntSet()
+	visibility := newSelectionSet()
 	it := preselection.Iterator()
 	for it.Next() {
 		for _, f := range predicates {
@@ -926,7 +1135,7 @@ func (p *SelectionParser) parsePolyrange() selEvaluator {
 				ops = append(ops, op)
 			}
 		}
-		polyrange = func(x selEvalState, s *fastOrderedIntSet) *fastOrderedIntSet {
+		polyrange = func(x selEvalState, s selectionSet) selectionSet {
 			return p.imp().evalPolyrange(x, s, ops)
 		}
 	}
@@ -938,18 +1147,18 @@ const polyrangeDollar = maxInt
 
 // evalPolyrange evaluates a polyrange specification (list of intervals)
 func (p *SelectionParser) evalPolyrange(state selEvalState,
-	preselection *fastOrderedIntSet, ops []selEvaluator) *fastOrderedIntSet {
+	preselection selectionSet, ops []selEvaluator) selectionSet {
 	// preselection is not used since it is perfectly legal to have range
 	// bounds be outside of the reduced set.
-	selection := newFastOrderedIntSet()
+	selection := newSelectionSet()
 	for _, op := range ops {
 		sel := op(state, preselection)
-		if sel != nil {
+		if sel.isDefined() {
 			selection = selection.Union(sel)
 		}
 	}
 	// Resolve spans
-	resolved := newFastOrderedIntSet()
+	resolved := newSelectionSet()
 	last := minInt
 	spanning := false
 	it := selection.Iterator()
@@ -1002,20 +1211,20 @@ func (p *SelectionParser) parseAtom() selEvaluator {
 		if err != nil {
 			panic(throw("command", "Atoi(%q) failed: %v", match, err))
 		}
-		op = func(selEvalState, *fastOrderedIntSet) *fastOrderedIntSet {
-			return newFastOrderedIntSet(number - 1)
+		op = func(selEvalState, selectionSet) selectionSet {
+			return newSelectionSet(number - 1)
 		}
 		p.line = p.line[len(match):]
 	} else if p.peek() == '$' { // $ means last commit, a la ed(1).
-		op = func(selEvalState, *fastOrderedIntSet) *fastOrderedIntSet {
-			return newFastOrderedIntSet(polyrangeDollar)
+		op = func(selEvalState, selectionSet) selectionSet {
+			return newSelectionSet(polyrangeDollar)
 		}
 		p.pop()
 	} else if p.peek() == ',' { // Comma just delimits a location spec
 		p.pop()
 	} else if strings.HasPrefix(p.line, "..") { // Following ".." means a span
-		op = func(selEvalState, *fastOrderedIntSet) *fastOrderedIntSet {
-			return newFastOrderedIntSet(polyrangeRange)
+		op = func(selEvalState, selectionSet) selectionSet {
+			return newSelectionSet(polyrangeRange)
 		}
 		p.line = p.line[len(".."):]
 	} else if p.peek() == '.' {
@@ -1028,7 +1237,7 @@ func (p *SelectionParser) parseAtom() selEvaluator {
 func (p *SelectionParser) parseTextSearch() selEvaluator {
 	p.eatWS()
 	type textSearcher interface {
-		evalTextSearch(selEvalState, *fastOrderedIntSet, *regexp.Regexp, string) *fastOrderedIntSet
+		evalTextSearch(selEvalState, selectionSet, *regexp.Regexp, string) selectionSet
 	}
 	searcher, ok := p.subclass.(textSearcher)
 	if !ok {
@@ -1054,7 +1263,7 @@ func (p *SelectionParser) parseTextSearch() selEvaluator {
 		for x := range seen {
 			modifiers.WriteRune(x)
 		}
-		return func(x selEvalState, s *fastOrderedIntSet) *fastOrderedIntSet {
+		return func(x selEvalState, s selectionSet) selectionSet {
 			return searcher.evalTextSearch(x, s, re, modifiers.String())
 		}
 	}
@@ -1107,7 +1316,7 @@ func (p *SelectionParser) parseFuncall() selEvaluator {
 	if op == nil {
 		panic(throw("command", "no such function @%s()", funname.String()))
 	}
-	return func(x selEvalState, s *fastOrderedIntSet) *fastOrderedIntSet {
+	return func(x selEvalState, s selectionSet) selectionSet {
 		return op(x, subarg(x, s))
 	}
 }
@@ -1122,7 +1331,7 @@ var selFuncs = map[string]selEvaluator{
 	"rev": revHandler,
 }
 
-func selMin(s *fastOrderedIntSet) int {
+func selMin(s selectionSet) int {
 	if s.Size() == 0 {
 		panic(throw("command", "cannot take minimum of empty set"))
 	}
@@ -1137,7 +1346,7 @@ func selMin(s *fastOrderedIntSet) int {
 	return n
 }
 
-func selMax(s *fastOrderedIntSet) int {
+func selMax(s selectionSet) int {
 	if s.Size() == 0 {
 		panic(throw("command", "cannot take maximum of empty set"))
 	}
@@ -1153,26 +1362,26 @@ func selMax(s *fastOrderedIntSet) int {
 }
 
 // Minimum member of a selection set.
-func minHandler(state selEvalState, subarg *fastOrderedIntSet) *fastOrderedIntSet {
-	return newFastOrderedIntSet(selMin(subarg))
+func minHandler(state selEvalState, subarg selectionSet) selectionSet {
+	return newSelectionSet(selMin(subarg))
 }
 
 // Maximum member of a selection set.
-func maxHandler(state selEvalState, subarg *fastOrderedIntSet) *fastOrderedIntSet {
-	return newFastOrderedIntSet(selMax(subarg))
+func maxHandler(state selEvalState, subarg selectionSet) selectionSet {
+	return newSelectionSet(selMax(subarg))
 }
 
 // Amplify - map empty set to empty, nonempty set to all.
-func ampHandler(state selEvalState, subarg *fastOrderedIntSet) *fastOrderedIntSet {
+func ampHandler(state selEvalState, subarg selectionSet) selectionSet {
 	if subarg.Size() != 0 {
 		return state.allItems()
 	}
-	return newFastOrderedIntSet()
+	return newSelectionSet()
 }
 
 // Predecessors function; all elements previous to argument set.
-func preHandler(state selEvalState, subarg *fastOrderedIntSet) *fastOrderedIntSet {
-	pre := newFastOrderedIntSet()
+func preHandler(state selEvalState, subarg selectionSet) selectionSet {
+	pre := newSelectionSet()
 	if subarg.Size() == 0 {
 		return pre
 	}
@@ -1187,8 +1396,8 @@ func preHandler(state selEvalState, subarg *fastOrderedIntSet) *fastOrderedIntSe
 }
 
 // Successors function; all elements following argument set.
-func sucHandler(state selEvalState, subarg *fastOrderedIntSet) *fastOrderedIntSet {
-	suc := newFastOrderedIntSet()
+func sucHandler(state selEvalState, subarg selectionSet) selectionSet {
+	suc := newSelectionSet()
 	if subarg.Size() == 0 {
 		return suc
 	}
@@ -1204,19 +1413,20 @@ func sucHandler(state selEvalState, subarg *fastOrderedIntSet) *fastOrderedIntSe
 }
 
 // Sort the argument set.
-func srtHandler(state selEvalState, subarg *fastOrderedIntSet) *fastOrderedIntSet {
-	return subarg.Sort()
+func srtHandler(state selEvalState, subarg selectionSet) selectionSet {
+	subarg.Sort()
+	return subarg
 }
 
 // Reverse the argument set.
-func revHandler(state selEvalState, subarg *fastOrderedIntSet) *fastOrderedIntSet {
+func revHandler(state selEvalState, subarg selectionSet) selectionSet {
 	n := subarg.Size()
 	v := make([]int, n)
 	it := subarg.Iterator()
 	for it.Next() {
 		v[n-it.Index()-1] = it.Value()
 	}
-	return newFastOrderedIntSet(v...)
+	return newSelectionSet(v...)
 }
 
 type attrEditAttr interface {
@@ -1361,8 +1571,8 @@ func (p *attrEditSelParser) visibilityTypeletters() map[rune]func(int) bool {
 }
 
 func (p *attrEditSelParser) evalTextSearch(state selEvalState,
-	preselection *fastOrderedIntSet,
-	search *regexp.Regexp, modifiers string) *fastOrderedIntSet {
+	preselection selectionSet,
+	search *regexp.Regexp, modifiers string) selectionSet {
 	var checkName, checkEmail bool
 	if len(modifiers) == 0 {
 		checkName, checkEmail = true, true
@@ -1377,7 +1587,7 @@ func (p *attrEditSelParser) evalTextSearch(state selEvalState,
 			}
 		}
 	}
-	found := newFastOrderedIntSet()
+	found := newSelectionSet()
 	it := preselection.Iterator()
 	for it.Next() {
 		a := p.attributions[it.Value()]
@@ -1391,12 +1601,12 @@ func (p *attrEditSelParser) evalTextSearch(state selEvalState,
 
 // AttributionEditor inspects and edits committer, author, tagger attributions
 type AttributionEditor struct {
-	eventSel []int
+	eventSel selectionSet
 	events   []Event
-	machine  func([]attrEditAttr) []int
+	machine  func([]attrEditAttr) selectionSet
 }
 
-func newAttributionEditor(sel []int, events []Event, machine func([]attrEditAttr) []int) *AttributionEditor {
+func newAttributionEditor(sel selectionSet, events []Event, machine func([]attrEditAttr) selectionSet) *AttributionEditor {
 	return &AttributionEditor{sel, events, machine}
 }
 
@@ -1434,11 +1644,12 @@ func (p *AttributionEditor) getMark(e Event) string {
 	return m
 }
 
-func (p *AttributionEditor) apply(f func(p *AttributionEditor, eventNo int, e Event, attrs []attrEditAttr, sel []int, extra ...interface{}), extra ...interface{}) {
-	for _, i := range p.eventSel {
+func (p *AttributionEditor) apply(f func(p *AttributionEditor, eventNo int, e Event, attrs []attrEditAttr, sel selectionSet, extra ...interface{}), extra ...interface{}) {
+	for it := p.eventSel.Iterator(); it.Next(); {
+		i := it.Value()
 		e := p.events[i]
 		attrs := p.attributions(e)
-		var sel []int
+		var sel selectionSet
 		func() {
 			defer func() {
 				if x := catch("command", recover()); x != nil {
@@ -1521,16 +1732,17 @@ func (p *AttributionEditor) inspect(w io.Writer) {
 	p.apply((*AttributionEditor).doInspect, w)
 }
 
-func (p *AttributionEditor) doInspect(eventNo int, e Event, attrs []attrEditAttr, sel []int, extra ...interface{}) {
+func (p *AttributionEditor) doInspect(eventNo int, e Event, attrs []attrEditAttr, sel selectionSet, extra ...interface{}) {
 	w := extra[0].(io.Writer)
 	mark := p.getMark(e)
-	if sel == nil {
-		sel = make([]int, len(attrs))
+	if !sel.isDefined() {
+		sel = newSelectionSet()
 		for i := range attrs {
-			sel[i] = i
+			sel.Add(i)
 		}
 	}
-	for _, i := range sel {
+	for it := sel.Iterator(); it.Next(); {
+		i := it.Value()
 		a := attrs[i]
 		io.WriteString(w, fmt.Sprintf("%6d %6s %2d:%-9s %s\n", eventNo+1, mark, i+1, a.desc(), a))
 	}
@@ -1541,15 +1753,15 @@ func (p *AttributionEditor) assign(args []string) {
 	p.apply((*AttributionEditor).doAssign, name, email, date)
 }
 
-func (p *AttributionEditor) doAssign(eventNo int, e Event, attrs []attrEditAttr, sel []int, extra ...interface{}) {
+func (p *AttributionEditor) doAssign(eventNo int, e Event, attrs []attrEditAttr, sel selectionSet, extra ...interface{}) {
 	name := extra[0].(string)
 	email := extra[1].(string)
 	date := extra[2].(Date)
-	if sel == nil {
+	if !sel.isDefined() {
 		panic(throw("command", "no attribution selected"))
 	}
-	for _, i := range sel {
-		attrs[i].assign(name, email, date)
+	for it := sel.Iterator(); it.Next(); {
+		attrs[it.Value()].assign(name, email, date)
 	}
 }
 
@@ -1557,12 +1769,12 @@ func (p *AttributionEditor) remove() {
 	p.apply((*AttributionEditor).doRemove)
 }
 
-func (p *AttributionEditor) doRemove(eventNo int, e Event, attrs []attrEditAttr, sel []int, extra ...interface{}) {
-	if sel == nil {
-		sel = p.authorIndices(attrs)
+func (p *AttributionEditor) doRemove(eventNo int, e Event, attrs []attrEditAttr, sel selectionSet, extra ...interface{}) {
+	if !sel.isDefined() {
+		sel = newSelectionSet(p.authorIndices(attrs)...)
 	}
-	rev := make([]int, len(sel))
-	copy(rev, sel)
+	rev := make([]int, sel.Size())
+	copy(rev, sel.Values())
 	sort.Stable(sort.Reverse(sort.IntSlice(rev)))
 	for _, i := range rev {
 		attrs[i].remove(e)
@@ -1574,20 +1786,20 @@ func (p *AttributionEditor) insert(args []string, after bool) {
 	p.apply((*AttributionEditor).doInsert, after, name, email, date)
 }
 
-func (p *AttributionEditor) doInsert(eventNo int, e Event, attrs []attrEditAttr, sel []int, extra ...interface{}) {
+func (p *AttributionEditor) doInsert(eventNo int, e Event, attrs []attrEditAttr, sel selectionSet, extra ...interface{}) {
 	after := extra[0].(bool)
 	name := extra[1].(string)
 	email := extra[2].(string)
 	date := extra[3].(Date)
-	if sel == nil {
-		sel = p.authorIndices(attrs)
+	if !sel.isDefined() {
+		sel = newSelectionSet(p.authorIndices(attrs)...)
 	}
 	var basis = -1
-	if len(sel) != 0 {
+	if sel.Size() != 0 {
 		if after {
-			basis = sel[len(sel)-1]
+			basis = sel.Fetch(sel.Size() - 1)
 		} else {
-			basis = sel[0]
+			basis = sel.Fetch(0)
 		}
 	}
 	var a Attribution
@@ -1606,22 +1818,22 @@ func (p *AttributionEditor) resolve(w io.Writer, label string) {
 	p.apply((*AttributionEditor).doResolve, w, label)
 }
 
-func (p *AttributionEditor) doResolve(eventNo int, e Event, attrs []attrEditAttr, sel []int, extra ...interface{}) {
+func (p *AttributionEditor) doResolve(eventNo int, e Event, attrs []attrEditAttr, sel selectionSet, extra ...interface{}) {
 	w := extra[0].(io.Writer)
 	label := extra[1].(string)
-	if sel == nil {
-		panic(throw("command", "no attribution selected"))
-	}
+	//if sel == nil {
+	//	panic(throw("command", "no attribution selected"))
+	//}
 	var b strings.Builder
 	if len(label) != 0 {
 		b.WriteString(fmt.Sprintf("%s: ", label))
 	}
 	b.WriteString(fmt.Sprintf("%6d %6s [", eventNo+1, p.getMark(e)))
-	for i, n := range sel {
-		if i > 0 {
+	for it := sel.Iterator(); it.Next(); {
+		if it.Index() > 0 {
 			b.WriteString(", ")
 		}
-		b.WriteString(strconv.Itoa(n + 1))
+		b.WriteString(strconv.Itoa(it.Value() + 1))
 	}
 	b.WriteString("]\n")
 	io.WriteString(w, b.String())
