@@ -1,4 +1,4 @@
 #!/bin/sh
 ## Test reduce subcommand to topologically reduce a repo
-${REPOCUTTER:-repocutter} -q reduce debranch.svn 
+${REPOCUTTER:-repocutter} -q reduce <debranch.svn 
 
