@@ -5,9 +5,11 @@ package main
 // SPDX-License-Identifier: BSD-2-Clause
 
 import (
+	"bufio"
 	"bytes"
 	"flag"
 	"fmt"
+	"io"
 	"io/ioutil"
 	"log"
 	"os"
@@ -15,7 +17,9 @@ import (
 	"path"
 	"path/filepath"
 	"regexp"
+
 	//"sort"
+	"strconv"
 	"strings"
 	"text/template"
 	"time"
@@ -60,7 +64,7 @@ var makefileTemplate = `# Makefile for {{.Project}} conversion using reposurgeon
 # 1. Make sure reposurgeon and repotool are on your $PATH.
 # 2. (Skip this step if you're starting from a stream file.) For svn, set
 #    REMOTE_URL to point at the remote repository you want to convert;
-#    you can use either an svn: URL or an rsync: UIRL for this.
+#    you can use either an svn: URL or an rsync: URL for this.
 #    If the repository is already in a DVCS such as hg or git,
 #    set REMOTE_URL to either the normal cloning URL (starting with hg://,
 #    git://, etc.) or to the path of a local clone.
@@ -81,17 +85,11 @@ var makefileTemplate = `# Makefile for {{.Project}} conversion using reposurgeon
 # 6. Run 'make stubmap' to create a stub author map.
 # 7. Run 'make' to build a converted repository.
 #
-# The reason both first- and second-stage stream files are generated is that,
-# especially with Subversion, making the first-stage stream file is often
-# painfully slow. By splitting the process, we lower the overhead of
-# experiments with the lift script.
-#
 # For a production-quality conversion you will need to edit the map
 # file and the lift script.  During the process you can set EXTRAS to
 # name extra metadata such as a comments message-box.
 #
-# Afterwards, you can use the headcompare and tagscompare productions
-# to check your work.
+# Afterwards, you can use the *compare productions to check your work.
 #
 
 EXTRAS = 
@@ -107,12 +105,13 @@ VERBOSITY = "set progress"
 REPOSURGEON = reposurgeon
 LOGFILE = conversion.log
 
+# Set and uncomment these if remote access tio Subversion needs credentials.
+#export RUSERNAME=
+#export RPASSWORD=
+
 # Configuration ends here
 
 .PHONY: local-clobber remote-clobber gitk gc compare clean dist stubmap
-# Tell make not to auto-remove tag directories, because it only tries rm 
-# and hence fails
-.PRECIOUS: {{.Project}}-%-checkout {{.Project}}-%-{{.TargetVCS}}
 
 default: {{.Project}}-{{.TargetVCS}}
 
@@ -129,20 +128,16 @@ default: {{.Project}}-{{.TargetVCS}}
 	repotool mirror $(REMOTE_URL) {{.Project}}-mirror
 
 # Make a local checkout of the source mirror for inspection
-{{.Project}}-checkout: {{.Project}}-mirror
-	cd {{.Project}}-mirror >/dev/null; repotool checkout $(PWD)/{{.Project}}-checkout
+%-checkout: %-mirror
+	cd %-mirror >/dev/null; repotool checkout $(PWD)/%-checkout
 
-# Make a local checkout of the source mirror for inspection at a specific tag
-{{.Project}}-%-checkout: {{.Project}}-mirror
-	cd {{.Project}}-mirror >/dev/null; repotool checkout $(PWD)/{{.Project}}-$*-checkout $*
-
-# Force rebuild of first-stage stream from the local mirror on the next make
+# Force rebuild of stream from the local mirror on the next make
 local-clobber: clean
-	rm -fr {{.Project}}.fi {{.Project}}-{{.TargetVCS}} *~ .rs* {{.Project}}-conversion.tar.gz {{.Project}}-*-{{.TargetVCS}}
+	rm -fr {{.Project}}.fi {{.Project}}-{{.TargetVCS}}
 
 # Force full rebuild from the remote repo on the next make.
 remote-clobber: local-clobber
-	rm -fr {{.Project}}.{{.SourceVCS}} {{.Project}}-mirror {{.Project}}-checkout {{.Project}}-*-checkout
+	rm -fr {{.Project}}.{{.SourceVCS}} *-mirror *-checkout
 
 # Get the (empty) state of the author mapping from the first-stage stream
 stubmap: {{.Project}}.{{.SourceVCS}}
@@ -164,7 +159,7 @@ clean:
 	rm -fr *~ .rs* {{.Project}}-conversion.tar.gz *.{{.SourceVCS}} *.fi *.fo
 
 # Bundle up the conversion metadata for shipping
-SOURCES = Makefile {{.Project}}.lift {{.Project}}.map $(EXTRAS)
+SOURCES = Makefile {{.Project}}.opts {{.Project}}.lift {{.Project}}.map $(EXTRAS)
 {{.Project}}-conversion.tar.gz: $(SOURCES)
 	tar --dereference --transform 's:^:{{.Project}}-conversion/:' -czvf {{.Project}}-conversion.tar.gz $(SOURCES)
 
@@ -241,6 +236,34 @@ func runShellProcessOrDie(dcmd string, legend string) {
 	}
 }
 
+// Either execute a command with line-by-line monitoring of output or die noisily
+func runMonitoredProcessOrDie(dcmd string, legend string, hook func(string)) {
+	if legend != "" {
+		legend = " " + legend
+	}
+	if verbose {
+		announce("monitoring '%s'%s", dcmd, legend)
+	}
+	cmd := exec.Command("sh", "-c", "("+dcmd+")")
+	pipeReader, pipeWriter := io.Pipe()
+	cmd.Stdin = os.Stdin
+	cmd.Stdout = pipeWriter
+	cmd.Stderr = os.Stderr
+
+	scanner := bufio.NewScanner(pipeReader)
+	scanner.Split(bufio.ScanLines)
+	go func() {
+		for scanner.Scan() {
+			hook(scanner.Text())
+		}
+	}()
+
+	err := cmd.Run()
+	if err != nil {
+		croak("executing %q: %v", dcmd, err)
+	}
+}
+
 // captureFromProcess runs a specified command, capturing the output.
 func captureFromProcess(command string, legend string) string {
 	if verbose {
@@ -257,21 +280,6 @@ func captureFromProcess(command string, legend string) string {
 	return string(content)
 }
 
-func exists(pathname string) bool {
-	_, err := os.Stat(pathname)
-	return !os.IsNotExist(err)
-}
-
-func isdir(pathname string) bool {
-	st, err := os.Stat(pathname)
-	return err == nil && st.Mode().IsDir()
-}
-
-func islink(pathname string) bool {
-	st, err := os.Stat(pathname)
-	return err == nil && (st.Mode()&os.ModeSymlink) != 0
-}
-
 func under(target string, hook func()) {
 	if verbose {
 		fmt.Printf("repotool: in %s...\n", target)
@@ -434,7 +442,7 @@ func export() {
 
 func mirror(args []string) {
 	if verbose {
-		fmt.Printf("initialize args: %v\n", args)
+		fmt.Printf("mirror args: %v\n", args)
 	}
 	operand := args[0]
 	mirrordir := ""
@@ -455,6 +463,26 @@ func mirror(args []string) {
 	}
 	username := os.Getenv("RUSERNAME")
 	password := os.Getenv("RPASSWORD")
+	plausibleSVNPrefix := func(operand string) bool {
+		return strings.HasPrefix(operand, "svn://") || strings.HasPrefix(operand, "svn+ssh://") || strings.HasPrefix(operand, "file://") || strings.HasPrefix(operand, "https://") || strings.HasPrefix(operand, "http://")
+	}
+	// Gets commit length of a Subversion repo from its URL. Can be run
+	// on a local mirror directory.
+	reposize := func(operand string) int {
+		if !plausibleSVNPrefix(operand) {
+			operand = "file://" + operand
+		}
+		infoCredentials := ""
+		if username != "" {
+			infoCredentials = fmt.Sprintf("--username %q", username)
+		}
+		if password != "" {
+			infoCredentials += fmt.Sprintf(" --password %q", password)
+		}
+		s := captureFromProcess(fmt.Sprintf("svn info --show-item=revision %s %s", infoCredentials, operand), "info")
+		revs, _ := strconv.Atoi(strings.Trim(s, "\n"))
+		return revs
+	}
 	var locald string
 	tillHash := regexp.MustCompile("^.*#")
 	isFullURL, badre := regexp.Match("svn://|svn\\+ssh://|https://|http://", []byte(operand))
@@ -466,12 +494,12 @@ func mirror(args []string) {
 		} else {
 			locald = filepath.Join(pwd, mirrordir)
 		}
-		credentials := ""
+		mirrorCredentials := ""
 		if username != "" {
-			credentials = fmt.Sprintf("--source-username %q", username)
+			mirrorCredentials = fmt.Sprintf("--source-username %q", username)
 		}
 		if password != "" {
-			credentials += fmt.Sprintf(" --source-password %q", username)
+			mirrorCredentials += fmt.Sprintf(" --source-password %q", password)
 		}
 		runShellProcessOrDie("svnadmin create "+locald, "mirror creation")
 		makeStub(locald+"/hooks/pre-revprop-change", "#!/bin/sh\nexit 0;\n")
@@ -483,15 +511,59 @@ func mirror(args []string) {
 		// very sure you have not made any local changes to the repo
 		// since rsyncing, or havoc will ensue.
 		runShellProcessOrDie(fmt.Sprintf("chmod a+x %s/hooks/pre-revprop-change", locald), "mirroring")
-		runShellProcessOrDie(fmt.Sprintf("svnsync init -q %s --allow-non-empty file://%s %s", credentials, locald, operand), "mirroring")
-		runShellProcessOrDie(fmt.Sprintf("svnsync synchronize -q --steal-lock file://%s", locald), "mirroring")
+		runShellProcessOrDie(fmt.Sprintf("svnsync init -q %s --allow-non-empty file://%s %s", mirrorCredentials, locald, operand), "mirroring")
+		baton := newBaton(!quiet, func(s string) {})
+		baton.startProgress("Mirroring", uint64(reposize(operand)))
+		cmd := fmt.Sprintf("svnsync synchronize %s --steal-lock file://%s", mirrorCredentials, locald)
+		ind := 0
+		runMonitoredProcessOrDie(cmd, "mirroring", func(line string) {
+			if strings.Contains(line, "Committed revision") {
+				ind++
+				baton.percentProgress(uint64(ind))
+			}
+		})
+		if !quiet {
+			baton.Write([]byte{'\n'}) // Kludge, FIXME
+		}
+		baton.endProgress()
 	} else if isdir(filepath.Join(operand, "locks")) {
 		if operand[0] == os.PathSeparator {
 			locald = operand
 		} else {
 			locald = filepath.Join(pwd, operand)
 		}
-		runShellProcessOrDie(fmt.Sprintf("svnsync synchronize -q --steal-lock file://%s", locald), "mirroring")
+		mirrorCredentials := ""
+		if username != "" {
+			mirrorCredentials = fmt.Sprintf("--source-username %q", username)
+		}
+		if password != "" {
+			mirrorCredentials += fmt.Sprintf(" --source-password %q", password)
+		}
+		getremote := fmt.Sprintf("svnlook pg %s -r 0 --revprop svn:sync-from-url", operand)
+		cmd := fmt.Sprintf("svnsync synchronize -q --steal-lock %s file://%s", mirrorCredentials, locald)
+		if remote := captureFromProcess(getremote, "getting remote URL"); !plausibleSVNPrefix(remote) {
+			// Without the remote size we can't progress-meter.
+			// Might happen if we rsynced this.
+			runShellProcessOrDie(cmd, "mirroring")
+		} else {
+			// Have remote size, we can progress-meter,
+			// this makes long resyncs more bearable.
+			baton := newBaton(!quiet, func(s string) {})
+			remotesize := reposize(remote)
+			localsize := reposize(locald)
+			baton.startProgress("Mirroring", uint64(remotesize-localsize))
+			ind := 0
+			runMonitoredProcessOrDie(cmd, "mirroring", func(line string) {
+				if strings.Contains(line, "Committed revision") {
+					ind++
+					baton.percentProgress(uint64(ind))
+				}
+			})
+			if !quiet {
+				baton.Write([]byte{'\n'}) // Kludge, FIXME
+			}
+			baton.endProgress()
+		}
 	} else if strings.HasPrefix(operand, "rsync://") {
 		if mirrordir == "" {
 			locald = filepath.Join(pwd, filepath.Base(operand)+"-mirror")
@@ -627,9 +699,18 @@ func checkout(outdir string, rev string) string {
 		if rev != "" {
 			rev = "-r " + rev
 		}
+		/* These options aren't actually documnted */
+		username := os.Getenv("RUSERNAME")
+		password := os.Getenv("RPASSWORD")
+		if username != "" {
+			rev += fmt.Sprintf("--username %q", username)
+		}
+		if password != "" {
+			rev += fmt.Sprintf(" --password %q", password)
+		}
 		// The reason for checkout's odd calling signature -
 		// pass it a checkout directory, get back a symlink
-		// toi what you actually wanted - is here. The problem
+		// to what you actually wanted - is here. The problem
 		// is that Subversion checkoutd on large repositories
 		// are horribly slow.  In case we're doing a
 		// comparison on all tags and branches, we want to
@@ -928,7 +1009,7 @@ func compareRevision(args []string, rev string) string {
 			diff += fmt.Sprintf("Same: %s\n", path)
 		}
 
-		// Check for permission mismatch,  We have to skip directories beccause
+		// Check for permission mismatch,  We have to skip directories because
 		// of Go MkdirAll's behavior that requiring seek permission; this makes for
 		// spurious mismatches in the x permission bit. The error cases here
 		// can be reached by symlink entries in Subversion files.
